package premium_store.controller.controller;

import org.springframework.web.bind.annotation.*;
import premium_store.model.TankLevel;
import premium_store.service.LevelService;

import java.util.List;

//привязываем наш контроллер к придуманному корневому URL благодаря аннотациям
//здесь происходит внедрение зависимости нашего сервиса
//так же здесь прописываем вызовы методов CRUD в привязке к URL
@RestController
@RequestMapping("/level")
public class LevelController {
    private final LevelService levelService;

    public LevelController(LevelService levelService){
        this.levelService = levelService;
    }

    //аннотация PathVariable связывает значения id из URL и Long id
    @GetMapping("/{id}")
    public TankLevel getLevel(@PathVariable Long id) {
        return levelService.findLevel(id);
    }

    //с помощью Java Stream преобразуем набор пришедших данных в объекты StudentDto
    @GetMapping("/")
    public List<TankLevel> getLevels() {
        return levelService.findAllLevels().stream()
                .toList();
    }

    @PostMapping("/")
    public TankLevel createLevel(@RequestParam("Level") int level) {
        return levelService.addLevel(level);
    }

    @PutMapping("/{id}")
    public TankLevel updateLevel(@PathVariable Long id,
                                 @RequestParam("Level") int level) {
        return levelService.updateLevel(id, level);
    }

    @DeleteMapping("/{id}")
    public TankLevel deleteLevel(@PathVariable Long id) {
        return levelService.deleteLevel(id);
    }
}
