package premium_store.model;

import javax.persistence.*;
import java.util.Objects;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Entity
public class Tank {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;

    @ManyToOne
    @JoinColumn(name = "nation_id")
    private Nation nation;

    @ManyToOne
    @JoinColumn(name = "level_id")
    private TankLevel tankLevel;

    private int cost;

    //реализация двунаправленной связи многие-ко-многим
    @ManyToMany(fetch= FetchType.EAGER)
    @JoinTable(name = "tanks_of_clients",
            joinColumns = @JoinColumn(name = "tank_id"),
            inverseJoinColumns = @JoinColumn(name = "client_id"))
    private List<GameClient> gameClients;

    public Tank() {
    }

    public Tank(String name, Nation nation, TankLevel tankLevel, int cost) {
        this.name = name;
        this.nation = nation;
        this.tankLevel = tankLevel;
        this.cost = cost;
        gameClients = new ArrayList<>();
    }

    public Long getId(){
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }

    public TankLevel getLevel() {
        return tankLevel;
    }

    public void setLevel(TankLevel tankLevel) {
        this.tankLevel = tankLevel;
    }

    public Nation getNation() {
        return nation;
    }

    public void setNation(Nation nation) {
        this.nation = nation;
    }

    public List<GameClient> getClients() {
        return gameClients;
    }

    public void setClients(List<GameClient> gameClients) {
        this.gameClients = gameClients;
    }

    public void addClient(GameClient newGameClient){
        if  (gameClients == null){
            this.gameClients = new ArrayList<>();
        }

        if (!gameClients.contains(newGameClient))
            this.gameClients.add(newGameClient);
    }

    public void removeClient(GameClient remGameClient){
        if (gameClients.contains(remGameClient))
            this.gameClients.remove(remGameClient);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;

        if (!(o instanceof Tank tank)) return false;

        return Objects.equals(getId(), tank.getId()) && Objects.equals(getName(), tank.getName())
                && Objects.equals(getCost(), tank.getCost()) && Objects.equals(getClients(), tank.getClients())
                && Objects.equals(getLevel(), tank.getLevel());
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    //преобразование данных по объекту в строчку
    @Override
    public String toString() {
        return "Tank{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", nation='" + nation + '\'' +
                ", level='" + tankLevel + '\'' +
                ", cost='" + cost + '\'' +
                ", clients='" + gameClients.stream().map(Object::toString).collect(Collectors.joining(", ")) + '\'' +
                '}';
    }
}
